@@ -1,4 +1,4 @@
-(* $Id: ssl_exts.ml 919 2006-06-15 12:32:17Z gerd $ *)
+(* $Id: ssl_exts.ml 1170 2008-03-04 20:27:30Z gerd $ *)
 
 exception Shutdown_error of Ssl.ssl_error
 
@@ -22,9 +22,5 @@ external get_mode : Ssl.socket -> ssl_mode = "ocaml_ssl_get_mode"
 
 external set_mode : Ssl.socket -> ssl_mode -> unit = "ocaml_ssl_set_mode"
 
-external single_read : Ssl.socket -> string -> int -> int -> int
-  = "ocaml_ssl_read"
-
-external single_write : Ssl.socket -> string -> int -> int -> int 
-  = "ocaml_ssl_write"
-
+let single_read = Ssl.read
+let single_write = Ssl.write
