@@ -292,7 +292,13 @@ object(self)
     ajp_ch # flush()
 
   method close_out() =
-    ajp_ch # flush()
+    try
+      ajp_ch # flush()
+    with
+      | error ->
+	  Netlog.logf `Err
+	    "Netcgi_ajp: Suppressed error in close_out: %s"
+	    (Netexn.to_string error)
 
   method pos_out = pos_out
 
@@ -713,10 +719,11 @@ let handle_request ?script_name config output_type arg_store exn_handler f ~log
 	`Conn_error error
 
 
-let rec handle_connection fd ~config ?script_name output_type arg_store
+let rec handle_connection_1 fd ~config ?script_name output_type arg_store
     exn_handler f =
 
   let log = Some ajp_log_error in
+  let fd_style = Netsys.get_fd_style fd in
 
   (* FIXME: Although the debug info says the connection is
      recycled, apache does not close the sockect and tries to
@@ -725,8 +732,9 @@ let rec handle_connection fd ~config ?script_name output_type arg_store
      if I have to wait more and 1 sec for the next request. *)
   let cdir =
     try
-      let in_socks, _, _ =  Netsys.restarting_select [fd] [] [] 1. in
-      if in_socks = [] then (
+      let ok = Netsys.restart_tmo
+	(Netsys.wait_until_readable fd_style fd) 1.0 in
+      if not ok then (
 	(* ajp_log_error "Timeout waiting for the next connection.  Closing.";*)
 	`Conn_close
       )
@@ -737,15 +745,28 @@ let rec handle_connection fd ~config ?script_name output_type arg_store
 
   match cdir with
     | `Conn_keep_alive ->
-	handle_connection fd ~config ?script_name output_type arg_store
+	handle_connection_1 fd ~config ?script_name output_type arg_store
 	  exn_handler f
     | `Conn_close ->
+	Netlog.Debug.release_fd fd;
 	Unix.close fd
     | `Conn_error error ->
+	Netlog.Debug.release_fd fd;
 	Unix.close fd;
 	raise error
     (* other cdir not possible *)
 
+let handle_connection fd ~config ?script_name output_type arg_store
+    exn_handler f =
+  Netlog.Debug.track_fd
+    ~owner:"Netcgi_ajp"
+    ~descr:("connection from " ^
+	      try Netsys.string_of_sockaddr(Netsys.getpeername fd)
+	      with _ -> "(noaddr)")
+    fd;
+  handle_connection_1 fd ~config ?script_name output_type arg_store
+    exn_handler f
+
 
 let run
     ?props
@@ -764,6 +785,10 @@ let run
   Unix.setsockopt sock Unix.SO_REUSEADDR true;
   Unix.bind sock sockaddr;
   Unix.listen sock 5;
+  Netlog.Debug.track_fd
+    ~owner:"Netcgi_ajp"
+    ~descr:("master port " ^ string_of_int port)
+    sock;
   while true do
     let (fd, server) = Unix.accept sock in
     try
@@ -773,6 +798,6 @@ let run
     with
       | e when config.default_exn_handler ->
 	  (* Any exception occurring here is fatal *)
-	  ajp_log_error("FATAL: " ^ Printexc.to_string e);
+	  ajp_log_error("FATAL: " ^ Netexn.to_string e);
   done
 
