@@ -304,7 +304,13 @@ object(self)
     fcgi_ch # flush()
 
   method close_out() =
-    fcgi_ch # flush()
+    try
+      fcgi_ch # flush()
+    with
+      | error ->
+	  Netlog.logf `Err
+	    "Netcgi_fcgi: Suppressed error in close_out: %s"
+	    (Netexn.to_string error);
     
   method pos_out = pos_out
 
@@ -573,7 +579,7 @@ end
 class closed_in_obj : Netchannels.in_obj_channel =
 object
   method input (_:string) (_:int) (_:int) = raise Netchannels.Closed_channel
-  method close_in () = raise Netchannels.Closed_channel
+  method close_in () = ()
   method pos_in = raise Netchannels.Closed_channel
   method really_input (_:string) (_:int) (_:int) =
     raise Netchannels.Closed_channel
@@ -711,7 +717,7 @@ let handle_request config
 (* [handle_connection fd .. f] handle an accept()ed connection,
    reading incoming records on the file descriptor [fd] and running
    [f] for each incoming request. *)
-let rec handle_connection fd ~max_conns ~external_server ~config
+let rec handle_connection_1 fd ~max_conns ~external_server ~config
     output_type arg_store exn_handler f =
   let fd_cmd =
     handle_request config output_type arg_store exn_handler f ~max_conns 
@@ -739,23 +745,37 @@ let rec handle_connection fd ~max_conns ~external_server ~config
 
   match fd_cmd' with
     | `Conn_close ->
-	Unix.shutdown fd Unix.SHUTDOWN_ALL;
+        (try Unix.shutdown fd Unix.SHUTDOWN_ALL with _ -> ());
+	Netlog.Debug.release_fd fd;
 	Unix.close fd
     | `Conn_close_linger ->
 	Unix.setsockopt_optint fd Unix.SO_LINGER (Some 15);
-	Unix.shutdown fd Unix.SHUTDOWN_ALL;
+        (try Unix.shutdown fd Unix.SHUTDOWN_ALL with _ -> ());
+	Netlog.Debug.release_fd fd;
 	Unix.close fd
     | `Conn_keep_alive ->
 	(* The server is supposed to take care of closing [fd].  (Tail
 	   recursiveness is important as many requests may be handled by
 	   this fun.)  *)
-	handle_connection fd ~external_server ~max_conns
+	handle_connection_1 fd ~external_server ~max_conns
 	  ~config output_type arg_store exn_handler f
     | `Conn_error e ->
-	Unix.shutdown fd Unix.SHUTDOWN_ALL;
+        (try Unix.shutdown fd Unix.SHUTDOWN_ALL with _ -> ());
+	Netlog.Debug.release_fd fd;
 	Unix.close fd;
 	raise e
 
+let handle_connection fd ~max_conns ~external_server ~config
+    output_type arg_store exn_handler f =
+  Netlog.Debug.track_fd 
+    ~owner:"Netcgi_fcgi"
+    ~descr:("connection from " ^ 
+	      try Netsys.string_of_sockaddr(Netsys.getpeername fd)
+	      with _ -> "(noaddr)")
+    fd;
+  handle_connection_1 fd ~max_conns ~external_server ~config
+    output_type arg_store exn_handler f
+
 
 let default_allow server =
   match server with
@@ -787,6 +807,10 @@ let run ?(config=Netcgi.default_config)
 	Unix.setsockopt sock Unix.SO_REUSEADDR true;
 	Unix.bind sock sockaddr;
 	Unix.listen sock 5;
+	Netlog.Debug.track_fd
+	  ~owner:"Netcgi_fcgi"
+	  ~descr:("master " ^ Netsys.string_of_sockaddr sockaddr)
+	  sock;
 	sock
   in
   let max_conns = 1 (* single process/thread *) in
