@@ -1,4 +1,4 @@
-(* $Id: netplex_container.mli 903 2006-06-04 18:10:32Z gerd $ *)
+(* $Id: netplex_container.mli 1256 2009-08-09 19:25:49Z gerd $ *)
 
 (** Containers
   *
@@ -15,3 +15,13 @@ val create_container :
 val create_admin_container : 
       Unixqueue.unix_event_system -> parallelization_type -> socket_service -> container
   (** {b Internally used.} The container for the special admin service *)
+
+module Debug : sig
+  val enable : bool ref
+    (** Enables debug logging of containers *)
+end
+
+(**/**)
+
+(* internal stuff: *)
+val string_of_event : Netplex_ctrl_aux.event -> string
