@@ -30,24 +30,28 @@ let protocol_of_string s =
   | (* Probably denial-of-service attack! *) None -> `Other
   
 type http_status =
+  (* 1xx: (informational) *)
   [
     | `Continue
     | `Switching_protocols
-    | `Ok
+    | (* 2xx: (successful) *)
+    `Ok
     | `Created
     | `Accepted
     | `Non_authoritative
     | `No_content
     | `Reset_content
     | `Partial_content
-    | `Multiple_choices
+    | (* 3xx: (redirection) *)
+    `Multiple_choices
     | `Moved_permanently
     | `Found
     | `See_other
     | `Not_modified
     | `Use_proxy
     | `Temporary_redirect
-    | `Bad_request
+    | (* 4xx: (client error) *)
+    `Bad_request
     | `Unauthorized
     | `Payment_required
     | `Forbidden
@@ -65,7 +69,8 @@ type http_status =
     | `Unsupported_media_type
     | `Requested_range_not_satisfiable
     | `Expectation_failed
-    | `Internal_server_error
+    | (* 5xx: (server error) *)
+    `Internal_server_error
     | `Not_implemented
     | `Bad_gateway
     | `Service_unavailable
@@ -73,11 +78,6 @@ type http_status =
     | `Http_version_not_supported
   ]
 
-(* 1xx: (informational) *)
-(* 2xx: (successful) *)
-(* 3xx: (redirection) *)
-(* 4xx: (client error) *)
-(* 5xx: (server error) *)
 let int_of_http_status =
   function
   | (* 1xx: (informational) *) `Continue -> 100
