@@ -1,4 +1,4 @@
-(* $Id: netsys_mem.ml 1654 2011-08-05 00:03:45Z gerd $ *)
+(* $Id: netsys_mem.ml 1716 2012-02-20 16:59:27Z gerd $ *)
 
 open Printf
 
@@ -34,6 +34,14 @@ let blit_string_to_memory s soff mem memoff len =
 external memory_address : memory -> nativeint
   = "netsys_memory_address"
 
+external reshape : 'a -> memory = "netsys_reshape"
+  (* 'a = any bigarray *)
+
+let memory_of_bigarray b = reshape b
+let memory_of_bigarray_1 b = reshape b
+let memory_of_bigarray_2 b = reshape b
+let memory_of_bigarray_3 b = reshape b
+
 external getpagesize : unit -> int
   = "netsys_getpagesize"
 
