@@ -1,4 +1,4 @@
-(* $Id: netsys_posix.ml 1629 2011-06-16 12:07:10Z gerd $ *)
+(* $Id: netsys_posix.ml 1742 2012-02-29 17:39:17Z gerd $ *)
 
 open Unix
 open Printf
@@ -39,6 +39,8 @@ external sysconf_open_max : unit -> int = "netsys_sysconf_open_max";;
 
 external fchdir : Unix.file_descr -> unit = "netsys_fchdir" ;;
 external fdopendir : Unix.file_descr -> Unix.dir_handle = "netsys_fdopendir" ;;
+external realpath : string -> string = "netsys_realpath"
+external get_nonblock : Unix.file_descr -> bool = "netsys_get_nonblock"
 
 (* Process groups, sessions, terminals *)
 
@@ -55,6 +57,12 @@ external ttyname : file_descr -> string = "netsys_ttyname";;
 
 external getsid : int -> int = "netsys_getsid";;
 
+external posix_openpt : bool -> Unix.file_descr = "netsys_posix_openpt"
+external grantpt : Unix.file_descr -> unit = "netsys_grantpt"
+external unlockpt : Unix.file_descr -> unit = "netsys_unlockpt"
+external ptsname : Unix.file_descr -> string = "netsys_ptsname"
+
+
 let with_tty f =
   let fd = 
     try Unix.openfile (ctermid()) [Unix.O_RDWR] 0 
@@ -131,6 +139,18 @@ let tty_read_password ?(tty=Unix.stdin) prompt =
 
 external setreuid : int -> int -> unit = "netsys_setreuid";;
 external setregid : int -> int -> unit = "netsys_setregid";;
+external initgroups : string -> int -> unit = "netsys_initgroups"
+
+(* mknod *)
+
+type node_type = 
+  | S_IFREG 
+  | S_IFCHR of int  (* major + minor *)
+  | S_IFBLK of int  (* major + minor *)
+  | S_IFIFO
+  | S_IFSOCK
+      
+external mknod : string -> int -> node_type -> unit = "netsys_mknod"
 
 (* poll *)
 
@@ -366,6 +386,276 @@ let int_of_act_events n = n
 let req_events_of_int n = n
 let int_of_req_events n = n
 
+(* poll aggregation *)
+
+type netsys_event_source
+type netsys_event_aggreg
+
+type tagged_event =
+  | EV_FD of Unix.file_descr
+
+type event_push =
+    { event_id : int;
+      event_tag : tagged_event;
+      event_api_req : int;
+    }
+
+class type event_source =
+object
+  method event_id : int
+  method event_tag : tagged_event
+  method push_record : event_push
+  method need_push : bool
+  method post_poll : int -> int -> unit
+  method upd_kernel_events : int -> unit
+  method set_post_modify_callback : (int -> unit) -> unit
+  method clear_post_modify_callback : unit -> unit
+  method modify_fd_event_source : int -> unit
+  method act_events : int
+end
+
+class fd_event_source fd req : event_source =
+  let event_id = ref 0 in
+  let event_api_req = ref req in
+  let event_kernel_req = ref 0 in
+  let event_tag = EV_FD fd in
+  let act_events = ref 0 in
+  let post_modify_callback = ref None in
+object(self)
+  initializer (
+    let id = Oo.id self in
+    event_id := id
+  )
+    
+  method event_id = !event_id
+  method event_tag = event_tag
+  method push_record =
+    { event_id = !event_id;
+      event_tag = event_tag;
+      event_api_req= !event_api_req
+    }
+  method need_push = !event_api_req <> !event_kernel_req
+  method upd_kernel_events req =
+    event_kernel_req := req;
+  method post_poll mask act = 
+    event_kernel_req := !event_kernel_req land mask;
+    act_events := act
+  method set_post_modify_callback f =
+    match !post_modify_callback with
+      | None -> post_modify_callback := Some f
+      | Some _ ->
+	  failwith "Netsys_posix.add_event_source: the source is already \
+                    added to an aggregator"
+  method clear_post_modify_callback () =
+    post_modify_callback := None
+  method modify_fd_event_source req =
+    event_api_req := req;
+    match !post_modify_callback with
+      | Some f -> f !event_id
+      | None -> ()
+  method act_events = !act_events
+end
+
+let fd_event_source fd req = 
+  new fd_event_source fd req
+
+let modify_fd_event_source es req =
+  es # modify_fd_event_source req
+
+let get_fd_of_event_source es =
+  match es # event_tag with
+    | EV_FD fd -> fd
+
+let act_events_of_event_source es =
+  es # act_events
+
+
+external have_event_aggregation : unit -> bool
+  = "netsys_have_event_aggregation"
+
+external netsys_create_event_aggreg :
+  bool -> netsys_event_aggreg
+  = "netsys_create_event_aggreg"
+
+external netsys_destroy_event_aggreg :
+  netsys_event_aggreg -> unit
+  = "netsys_destroy_event_aggreg"
+
+external netsys_event_aggreg_fd :
+  netsys_event_aggreg -> Unix.file_descr
+  = "netsys_event_aggreg_fd"
+
+external netsys_push_event_sources : 
+  netsys_event_aggreg -> event_push list -> unit
+  = "netsys_push_event_sources"
+
+external netsys_add_event_source :
+  netsys_event_aggreg -> event_push -> unit
+  = "netsys_add_event_source"
+
+external netsys_del_event_source :
+  netsys_event_aggreg -> int -> tagged_event -> unit
+  = "netsys_del_event_source"
+
+external netsys_poll_event_sources :
+  netsys_event_aggreg -> int -> (int * int * int) list
+  (* The list are triples (event_id, mask, act) *)
+  = "netsys_poll_event_sources"
+
+external netsys_interrupt_aggreg :
+  netsys_event_aggreg -> unit
+  = "netsys_interrupt_aggreg"
+
+class event_aggregator is_interruptible =
+  let up = ref true in
+  let all_sources = Hashtbl.create 27 in
+  let upd_sources = Hashtbl.create 27 in
+  let netsys = netsys_create_event_aggreg is_interruptible in
+  let check_up() =
+    if not !up then
+      failwith "Netsys_posix: This event_aggregator is already destroyed" in
+object(self)
+  method add_event_source (es : event_source) =
+    check_up();
+    es # set_post_modify_callback self#post_modify_callback;
+    try
+      netsys_add_event_source netsys es#push_record;
+      Hashtbl.replace all_sources es#event_id es;
+      Hashtbl.replace upd_sources es#event_id es
+    with
+      | error ->
+	  es # clear_post_modify_callback();
+	  raise error
+  method del_event_source (es : event_source) =
+    check_up();
+    if not(Hashtbl.mem all_sources es#event_id) then
+      failwith "Netsys_posix.del_event_source: not member of this aggregator";
+    netsys_del_event_source netsys es#event_id es#event_tag;
+    es # clear_post_modify_callback();
+    Hashtbl.remove all_sources es#event_id;
+    Hashtbl.remove upd_sources es#event_id
+  method post_modify_callback id =
+    check_up();
+    if (Hashtbl.mem all_sources id) then (
+      let es = Hashtbl.find all_sources id in
+      Hashtbl.replace upd_sources id es
+    )
+  method push_event_updates() =
+    check_up();
+    let l =
+      Hashtbl.fold 
+	(fun _ es acc -> 
+	   if es # need_push then
+	     let p = es#push_record in
+	     es # upd_kernel_events p.event_api_req;
+	     p :: acc
+	   else
+	     acc
+	) 
+	upd_sources
+	[] in
+    netsys_push_event_sources netsys l;
+    Hashtbl.clear upd_sources
+  method poll_event_sources tmo =
+    self # push_event_updates();
+    let v =
+      Netsys_impl_util.slice_time_ms
+	(fun millis ->
+	   let evpairs0 = netsys_poll_event_sources netsys millis in
+	   let evpairs1 = (* without spurious events *)
+	     List.filter
+	       (fun (id,_,_) ->
+		  Hashtbl.mem all_sources id
+	       )
+	       evpairs0 in
+	   let evpairs2 =
+	     List.map
+	       (fun (id,mask,act) ->
+		  let es = Hashtbl.find all_sources id in
+		  es#post_poll mask act;
+		  Hashtbl.replace upd_sources id es;
+		  es
+	       )
+	       evpairs1 in
+	   if evpairs2 = [] then
+	     None
+	   else
+	     Some evpairs2
+	)
+	tmo in
+    match v with
+      | None -> []
+      | Some l -> l
+  method event_aggregator_fd =
+    check_up();
+    netsys_event_aggreg_fd netsys
+  method interrupt_event_aggregator() =
+    netsys_interrupt_aggreg netsys
+  method destroy_event_aggregator () =
+    if !up then (
+      netsys_destroy_event_aggreg netsys;
+      up := false
+    )
+end
+
+let create_event_aggregator = new event_aggregator
+let add_event_source ea = ea # add_event_source
+let del_event_source ea = ea # del_event_source
+let push_event_updates ea = ea # push_event_updates()
+let poll_event_sources (ea : event_aggregator) = ea # poll_event_sources
+let event_aggregator_fd ea = ea # event_aggregator_fd
+let destroy_event_aggregator ea = ea # destroy_event_aggregator()
+let interrupt_event_aggregator ea = ea # interrupt_event_aggregator()
+
+
+(* events *)
+
+type not_event
+
+external nsys_create_event : bool -> not_event = "netsys_create_not_event"
+external set_nonblock_event : not_event -> unit = "netsys_set_nonblock_not_event"
+external get_event_fd : not_event -> Unix.file_descr = "netsys_get_not_event_fd"
+external set_event : not_event -> unit = "netsys_set_not_event"
+external wait_event : not_event -> unit = "netsys_wait_not_event"
+external consume_event : not_event -> unit = "netsys_consume_not_event"
+external nsys_destroy_event : not_event -> unit = "netsys_destroy_not_event"
+external nsys_return_all_event_fd : not_event -> Unix.file_descr list
+  = "netsys_return_all_not_event_fd"
+
+let create_event() = 
+  let e = nsys_create_event true in
+  List.iter
+    (fun fd ->
+       Netlog.Debug.track_fd
+	 ~owner:"Netsys_posix" 
+	 ~descr:"create_event"
+	 fd;
+    )
+    (nsys_return_all_event_fd e);
+  e
+
+let destroy_event e =
+  List.iter
+    (fun fd ->
+       Netlog.Debug.release_fd fd;
+    )
+    (nsys_return_all_event_fd e);
+  nsys_destroy_event e
+
+let report_signal_as_event ne signum =
+  (* This is simpler to implement in Ocaml than in C:
+     - [ne] is kept reachable all the time
+     - we can pass [ne] to the handler via a closure
+     - we can run non-async-signal-safe stuff in Ocaml signal handlers,
+       because they are actually called indirectly
+   *)
+  Sys.set_signal 
+    signum
+    (Sys.Signal_handle
+       (fun _ ->
+	  set_event ne
+       )
+    )
 
 (* post fork handlers *)
 
@@ -450,6 +740,50 @@ external mkfifoat : Unix.file_descr -> string -> int -> unit
 external readlinkat : Unix.file_descr -> string -> string
   = "netsys_readlinkat"
 
+(* Clocks *)
+
+type timespec = float * int
+type clock_id
+type clock =
+  | CLOCK_REALTIME
+  | CLOCK_MONOTONIC
+  | CLOCK_ID of clock_id
+
+external nanosleep : timespec -> timespec ref -> unit = "netsys_nanosleep"
+external clock_gettime : clock -> timespec = "netsys_clock_gettime"
+external clock_settime : clock -> timespec -> unit = "netsys_clock_settime"
+external clock_getres : clock -> timespec = "netsys_clock_getres"
+external clock_getcpuclockid : int -> clock_id = "netsys_clock_getcpuclockid"
+
+type timer_expiration =
+  | TEXP_NONE
+  | TEXP_EVENT of not_event
+  | TEXP_EVENT_CREATE
+  | TEXP_SIGNAL of int
+type posix_timer
+
+external have_posix_timer : unit -> bool
+  = "netsys_have_posix_timer"
+
+external nsys_timer_create : clock -> timer_expiration -> posix_timer
+  = "netsys_timer_create"
+
+external timer_settime : posix_timer -> bool -> timespec -> timespec -> unit
+  = "netsys_timer_settime"
+
+external timer_gettime : posix_timer -> timespec
+  = "netsys_timer_gettime"
+
+external timer_delete : posix_timer -> unit
+  = "netsys_timer_delete"
+
+external timer_event : posix_timer -> not_event
+  = "netsys_timer_event"
+
+let timer_create clock texp =
+  let v = nsys_timer_create clock texp in
+  Gc.finalise timer_delete v;
+  v
 
 (* Spawn *)
 
@@ -473,28 +807,51 @@ type fd_action =
 type sig_action =
   | Sig_default of int
   | Sig_ignore of int
+  | Sig_mask of int list
 
 external netsys_spawn : wd_spec -> pg_spec -> fd_action list -> 
                         sig_action list -> string array ->
                         string -> string array -> int
   = "netsys_spawn_byte" "netsys_spawn_nat"
 
-let spawn ?(chdir = Wd_keep) ?(pg = Pg_keep) ?(fd_actions = [])
-          ?(sig_actions = []) ?(env = Unix.environment()) cmd args =
-  (* Fixup: if pg = Pg_new_fg_group, we remove any Sig_default for
-     SIGTTOU from sig_actions. Because of special handling, the effect
-     of Sig_default is enforced by the implementation, but this must be
-     done at [execve] time.
-   *)
-  let sig_actions =
-    if pg = Pg_new_fg_group then
-      List.filter 
-	(fun spec -> spec <> Sig_default Sys.sigttou)
-	sig_actions
-    else
-      sig_actions in
-  netsys_spawn chdir pg fd_actions sig_actions env cmd args
+external netsys_posix_spawn : pg_spec -> fd_action list -> 
+                              sig_action list -> string array ->
+                              string -> string array -> int
+  = "netsys_posix_spawn_byte" "netsys_posix_spawn_nat"
 
+external have_posix_spawn : unit -> bool
+  = "netsys_have_posix_spawn"
+
+let spawn ?(chdir = Wd_keep) ?(pg = Pg_keep) ?(fd_actions = [])
+          ?(sig_actions = []) ?(env = Unix.environment()) 
+          ?(no_posix_spawn=false) cmd args =
+  (* Check whether we can use the faster netsys_posix_spawn *)
+  let use_posix_spawn =
+    not no_posix_spawn &&
+    have_posix_spawn() &&
+    chdir = Wd_keep &&
+    pg <> Pg_new_fg_group &&
+    not (List.exists
+	   (fun sa -> match sa with Sig_ignore _ -> true | _ -> false) 
+	   sig_actions) in
+  
+  if use_posix_spawn then
+    netsys_posix_spawn pg fd_actions sig_actions env cmd args
+  else (
+    (* Fixup: if pg = Pg_new_fg_group, we remove any Sig_default for
+       SIGTTOU from sig_actions. Because of special handling, the effect
+       of Sig_default is enforced by the implementation, but this must be
+       done at [execve] time.
+     *)
+    let sig_actions =
+      if pg = Pg_new_fg_group then
+	List.filter 
+	  (fun spec -> spec <> Sig_default Sys.sigttou)
+	  sig_actions
+      else
+	sig_actions in
+    netsys_spawn chdir pg fd_actions sig_actions env cmd args
+  )
 
 type watched_subprocess = 
     { atom_idx : int;
@@ -859,7 +1216,10 @@ let shm_create prefix size =
       let fd =
 	shm_open
 	  name [SHM_O_RDWR; SHM_O_CREAT; SHM_O_EXCL ] 0o600 in
-      Unix.fchmod fd 0o600;
+      ( try Unix.fchmod fd 0o600
+	with Unix.Unix_error(Unix.EINVAL,_,_) -> ()
+	  (* OSX seems to throw EINVAL here *)
+      );
       Unix.ftruncate fd size;
       (fd, name)
     with
