@@ -1,6 +1,246 @@
-(* $Id: netsys_posix.mli 1630 2011-06-16 12:10:01Z gerd $ *)
+(* $Id: netsys_posix.mli 1696 2012-02-08 19:27:53Z gerd $ *)
 
-(** POSIX-specific system calls missing in the [Unix] module *)
+(** POSIX-specific system calls missing in the [Unix] module, and
+    further API's from POSIX-style operating systems.
+ *)
+
+(** {1 Files, Processes, TTYs, Users, Groups} *)
+
+val int_of_file_descr : Unix.file_descr -> int
+  (** Return the file descriptor as integer. See also
+      {!Netsys.int64_of_file_descr} which works for all OS.
+   *)
+
+val file_descr_of_int : int -> Unix.file_descr
+  (** Make a file descriptor from an integer *)
+
+external sysconf_open_max : unit -> int = "netsys_sysconf_open_max"
+  (** Return the maximum number of open file descriptor per process.
+   * It is also ensured that for every file descriptor [fd]:
+   * [fd < sysconf_open_max()]
+   *)
+
+external get_nonblock : Unix.file_descr -> bool = "netsys_get_nonblock"
+  (** Returns whether the nonblock flag is set *)
+
+external fchdir : Unix.file_descr -> unit = "netsys_fchdir"
+  (** Set the current directory to the directory referenced by the
+      file descriptor
+   *)
+
+external fdopendir : Unix.file_descr -> Unix.dir_handle = "netsys_fdopendir"
+  (** Make a directory handle from a file descriptor. The descriptor
+      is then "owned" by the directory handle, and will be closed by
+      [Unix.closedir].
+
+      This function is useful in conjunction with {!Netsys_posix.openat}
+      to read directories relative to a parent directory.
+
+      This is a recent addition to the POSIX standard; be prepared to
+      get [Invalid_argument] because it is unavailable.
+   *)
+
+external realpath : string -> string = "netsys_realpath"
+  (** Returns a pathname pointing to the same filesystem object so that
+      the pathname does not include "." or ".." or symbolic links.
+   *)
+
+(* Process groups, sessions, terminals *)
+
+external getpgid : int -> int = "netsys_getpgid"
+  (** Return the process group ID of the process with the passed PID.
+   * For the number 0, the process group ID of the current process is
+   * returned.
+   *)
+
+val getpgrp : unit -> int
+  (** Same as [getpgid 0], i.e. returns the process group ID of the
+   * current process.
+   *)
+
+external setpgid : int -> int -> unit = "netsys_setpgid"
+  (** [setpgid pid pgid]: Set the process group ID of the process [pid]
+   * to [pgid]. If [pid = 0], the process group ID of the current process
+   * is changed. If [pgid = 0], as process group ID the process ID of the
+   * process referenced by [pid] is used.
+   *
+   * It is only possible for a process to join a process group if both
+   * belong to the same session.
+   *)
+
+val setpgrp : unit -> unit
+  (** Same as [setpgid 0 0]: A new process group ID is created, and the
+   * current process becomes its sole member.
+   *)
+
+external tcgetpgrp : Unix.file_descr -> int = "netsys_tcgetpgrp"
+  (** Return the process group ID of the foreground process group of
+   * the session associated with the file descriptor, which must be
+   * a tty.
+   *)
+
+external tcsetpgrp : Unix.file_descr -> int -> unit = "netsys_tcsetpgrp"
+  (** Sets the foreground process group ID of the session associated
+   * with the file descriptor, which must be a tty.
+   *)
+
+external ctermid : unit -> string = "netsys_ctermid"
+  (** Returns the name of the controlling tty of the current process 
+   * as pathname to a device file
+   *)
+
+external ttyname : Unix.file_descr -> string = "netsys_ttyname"
+  (** Returns the name of the controlling tty referred to by the
+   * file descriptor.
+   *)
+
+external getsid : int -> int = "netsys_getsid"
+  (** Returns the session ID of the process with the passed PID.
+   * For the PID 0, the session ID of the current process is returned.
+   *)
+
+val with_tty : (Unix.file_descr -> unit) -> unit
+  (** [with_tty f]: Runs [f fd] where [fd] is the terminal of the process.
+      If the process does not have a terminal (because it is a daemon) 
+      [with_tty] will fail.
+   *)
+
+val tty_read_password : ?tty:Unix.file_descr -> string -> string
+  (** [tty_read_password prompt]: If [tty] is a terminal, the [prompt]
+      is printed, and a password is read from the terminal (echo off).
+      If [tty] is not a terminal, no [prompt] is printed, and just a
+      line is read from the [tty] descriptor (non-interactive case).
+
+      [tty] defaults to [Unix.stdin]. If this function is used in a
+      program where stdin is not redirected, and the program is started
+      in a terminal, it will read the password with prompt and 
+      echo disabled. If stdin is redirected, it is assumed that the program is
+      used in a script, and the password is piped into it.
+
+      Use in conjunction with [with_tty] to ensure that [tty] is
+      the terminal even if a redirection is in effect, e.g.
+      {[ with_tty (fun tty -> tty_read_password ~tty prompt) ]}
+
+      Raises [Sys.Break] if the user triggers SIGINT (i.e. presses
+      CTRL-C) to abort the input of a password.
+   *)
+
+external posix_openpt : bool -> Unix.file_descr = "netsys_posix_openpt"
+  (** [posix_openpt noctty]: Opens an unused PTY master.
+
+      [noctty]: If true, the descriptor will not become the controlling
+      terminal.
+
+      If this function is not provided by the OS, an emulation is used.
+
+      On some OS, System V style PTY's are unavailable (but they get
+      rare).
+   *)
+
+external grantpt : Unix.file_descr -> unit = "netsys_grantpt"
+  (** Grant access to this PTY *)
+
+external unlockpt : Unix.file_descr -> unit = "netsys_unlockpt"
+  (** Unlock a PTY master/slave pair *)
+
+external ptsname : Unix.file_descr -> string = "netsys_ptsname"
+  (** Get the name of the slave PTY *)
+
+type node_type = 
+  | S_IFREG 
+  | S_IFCHR of int  (* major + minor *)
+  | S_IFBLK of int  (* major + minor *)
+  | S_IFIFO
+  | S_IFSOCK
+      
+external mknod : string -> int -> node_type -> unit = "netsys_mknod"
+    (** Creates the node with the given permissions and the given type *)
+
+
+(* Users and groups *)
+
+external setreuid : int -> int -> unit = "netsys_setreuid"
+  (** Changes both the real and the effective user ID of the current
+   * process.
+   *)
+
+external setregid : int -> int -> unit = "netsys_setregid"
+  (** Changes both the real and the effective group ID of the current
+   * process.
+   *)
+
+external initgroups : string -> int -> unit = "netsys_initgroups"
+  (** See initgroups(3). This is a non-POSIX function but widely
+      available.
+   *)
+
+
+(** {1 The "at" variants of system calls} *)
+
+(** Note that a few "at" calls have been omitted because the same
+    functionality can be achieved by first opening the file with
+    [openat] and then by using a function that references the file
+    by descriptor. An example for this is [fstatat]: After the
+    [openat] call one can use [fstat] to get the stat record of the
+    file.
+ *)
+
+val have_at : unit -> bool
+  (** Whether the [*at] functions are available (they were only recently
+      standardized and cannot be expected on all OS yet)
+   *)
+
+val at_fdcwd : Unix.file_descr
+  (** Pseudo descriptor value to be used as first argument of [*at]
+      functions
+   *)
+
+type at_flag = AT_EACCESS | AT_SYMLINK_NOFOLLOW | AT_REMOVEDIR
+  (** Flags one can pass to "at" functions. Not all functions support
+      all flags
+   *)
+
+val openat : Unix.file_descr -> string -> Unix.open_flag list -> 
+             Unix.file_perm -> 
+                Unix.file_descr
+  (** Same as [Unix.openfile] but open relative to the directory given
+      by first argument
+   *)
+
+val faccessat : Unix.file_descr -> string -> Unix.access_permission list ->
+                at_flag list ->
+                  unit
+  (** Same as [Unix.access] but the file is taken relative to the directory
+      given by first argument
+   *)
+
+val mkdirat : Unix.file_descr -> string -> int -> unit
+  (** Same as [Unix.mkdir] but the file is taken relative to the directory
+      given by first argument
+   *)
+ 
+val renameat : Unix.file_descr -> string -> Unix.file_descr -> string -> unit
+  (** [renameat olddirfd oldpath newdirfd newpath] *)
+
+val linkat : Unix.file_descr -> string -> Unix.file_descr -> string ->
+             at_flag list -> unit
+  (** [linkat olddirfd oldpath newdirfd newpath flags] *)
+
+val unlinkat : Unix.file_descr -> string -> at_flag list -> unit
+  (** Same as [Unix.unlink] but unlink the file relative to the directory
+      given by first argument
+   *)
+
+val symlinkat : string -> Unix.file_descr -> string -> unit
+  (** [symlinkat oldpath newdirfd newpath flags] *)
+
+val mkfifoat : Unix.file_descr -> string -> int -> unit
+  (** [mkfifoat dirfd path mode] *)
+
+val readlinkat : Unix.file_descr -> string -> string
+  (** [readlinkat dirfd path] *)
+
+(* TODO: futimens *)
 
 (** {1 File descriptor polling} *)
 
@@ -82,7 +322,7 @@ val poll_array_length : poll_array -> int
 
 val poll : poll_array -> int -> float -> int
   (** [poll a n tmo]: Poll for the events of the cells 0 to [n-1] of 
-      poll array [a], and set the [poll_revents] member of all cells.
+      poll array [a], and set the [poll_act_events] member of all cells.
       Wait for at most [tmo] seconds (a negative value means there is
       no timeout). Returns the number of ready file descriptors.
 
@@ -122,6 +362,111 @@ val const_hup_event : int
 val const_nval_event : int
 
 
+
+(** {1 Event aggregation} *)
+
+(** Support for "high-speed" poll implementations. Currently, only
+    [epoll] for Linux is supported.
+
+    The model exhibited in this API is the smallest common denominator
+    of Linux epoll, BSD kqueue, and Solaris ports. The [event_aggregator]
+    represents the set of monitored event sources. There is, so far,
+    only one source, namely file descriptors, i.e. one can check whether
+    a descriptor is readable or writable (like [poll]). The source can
+    be added to the [event_aggregator] to monitor the source.
+
+    By calling [poll_event_sources] one can determine sources that
+    are currently active (i.e. in signalling state). 
+
+    It is undefined what happens when a file descriptor is closed while
+    being member of the aggregator.
+ *)
+
+type event_aggregator
+type event_source
+
+val have_event_aggregation : unit -> bool
+  (** Whether there is an implementation for this OS *)
+
+val create_event_aggregator : bool -> event_aggregator
+  (** [create_event_aggregator is_interruptible]: Creates a new aggregator,
+      and allocates the required OS resources.
+
+      If [is_interruptible], the aggregator can be interrupted from a
+      different thread. See [interrupt_event_aggregator] below.
+   *)
+
+val destroy_event_aggregator : event_aggregator -> unit
+  (** Frees all OS resources *)
+
+val fd_event_source : Unix.file_descr -> poll_req_events -> event_source
+  (** Wraps a file descriptor as event_source, and monitors the
+      events in [poll_req_events].
+
+      The [event_source] contains
+      state about the relation to the aggregator, and because of this,
+      the [event_source] should only be used together with one aggregator
+      (at a time).
+   *)
+
+val modify_fd_event_source : event_source ->  poll_req_events -> unit
+  (** Modifies the set of events monitored at this event source *)
+
+val get_fd_of_event_source : event_source -> Unix.file_descr
+  (** Get the file descriptor wrapped by this event source *)
+
+val act_events_of_event_source : event_source -> poll_act_events
+  (** Return the actual events of the source. This is updated when
+      [poll_event_sources] returns the source.
+   *)
+
+val add_event_source : event_aggregator -> event_source ->  unit
+  (** Adds the event source to the aggregator *)
+
+val del_event_source : event_aggregator -> event_source ->  unit
+  (** Removes the source from the aggregator *)
+
+val interrupt_event_aggregator : event_aggregator -> unit
+  (** If [create_event_aggregator] was called with [true] as argument, the
+      aggregator is interruptible, and this function interrupts it. The
+      effect is that a currently running [poll_event_sources], or, if
+      it is not running, the next invocation of [poll_event_sources] 
+      returns immediately.
+
+      If the aggregator is not interruptible, this function is a NOP.
+   *)
+
+val push_event_updates : event_aggregator -> unit
+  (** Pushes all modifications of the sources to the kernel *)
+
+val poll_event_sources : event_aggregator -> float -> event_source list
+  (** [poll_event_sources ea tmo]: First, all modifications are pushed
+      to the kernel, and polling is set up to get events. If no events
+      can currently be delivered, the function waits up to [tmo] seconds
+      (or endlessly if negative) for events. The function returns only a
+      limited number of events at a time. It is allowed that the function 
+      returns fewer events than are currently in signalled state, even
+      none.
+
+      Call the function with [tmo=0.0] for non-blocking behavior.
+
+      Note that this is the "level-triggered" behavior: If a source
+      remains active it will be reported again by the next [poll_event_sources],
+      just as [poll] would do.
+   *)
+
+val event_aggregator_fd : event_aggregator -> Unix.file_descr
+  (** Returns the underlying file descriptor. It is implementation-defined
+      whether this descriptor can also be polled for events. Generally,
+      you should run [push_event_updates] before polling from the descriptor.
+   *)
+
+
+(* BSD: kqueue
+   Solaris: ports (port_create, port_associate)
+ *)
+
+
 (** {1 Fork helpers} *)
 
 (** Ocamlnet invokes [Unix.fork] at some places to create child processes
@@ -156,216 +501,35 @@ val run_post_fork_handlers : unit -> unit
    *)
 
 
-(** {1 The "at" variants of system calls} *)
-
-(** Note that a few "at" calls have been omitted because the same
-    functionality can be achieved by first opening the file with
-    [openat] and then by using a function that references the file
-    by descriptor. An example for this is [fstatat]: After the
-    [openat] call one can use [fstat] to get the stat record of the
-    file.
- *)
-
-val have_at : unit -> bool
-  (** Whether the [*at] functions are available (they were only recently
-      standardized and cannot be expected on all OS yet)
-   *)
-
-val at_fdcwd : Unix.file_descr
-  (** Pseudo descriptor value to be used as first argument of [*at]
-      functions
-   *)
-
-type at_flag = AT_EACCESS | AT_SYMLINK_NOFOLLOW | AT_REMOVEDIR
-  (** Flags one can pass to "at" functions. Not all functions support
-      all flags
-   *)
-
-val openat : Unix.file_descr -> string -> Unix.open_flag list -> 
-             Unix.file_perm -> 
-                Unix.file_descr
-  (** Same as [Unix.openfile] but open relative to the directory given
-      by first argument
-   *)
-
-val faccessat : Unix.file_descr -> string -> Unix.access_permission list ->
-                at_flag list ->
-                  unit
-  (** Same as [Unix.access] but the file is taken relative to the directory
-      given by first argument
-   *)
-
-val mkdirat : Unix.file_descr -> string -> int -> unit
-  (** Same as [Unix.mkdir] but the file is taken relative to the directory
-      given by first argument
-   *)
- 
-val renameat : Unix.file_descr -> string -> Unix.file_descr -> string -> unit
-  (** [renameat olddirfd oldpath newdirfd newpath] *)
-
-val linkat : Unix.file_descr -> string -> Unix.file_descr -> string ->
-             at_flag list -> unit
-  (** [linkat olddirfd oldpath newdirfd newpath flags] *)
-
-val unlinkat : Unix.file_descr -> string -> at_flag list -> unit
-  (** Same as [Unix.unlink] but unlink the file relative to the directory
-      given by first argument
-   *)
-
-val symlinkat : string -> Unix.file_descr -> string -> unit
-  (** [symlinkat oldpath newdirfd newpath flags] *)
-
-val mkfifoat : Unix.file_descr -> string -> int -> unit
-  (** [mkfifoat dirfd path mode] *)
-
-val readlinkat : Unix.file_descr -> string -> string
-  (** [readlinkat dirfd path] *)
-
-(* TODO: futimens *)
-
-(** {1 Misc} *)
-
-val int_of_file_descr : Unix.file_descr -> int
-  (** Return the file descriptor as integer. See also
-      {!Netsys.int64_of_file_descr} which works for all OS.
-   *)
-
-val file_descr_of_int : int -> Unix.file_descr
-  (** Make a file descriptor from an integer *)
-
-external sysconf_open_max : unit -> int = "netsys_sysconf_open_max"
-  (** Return the maximum number of open file descriptor per process.
-   * It is also ensured that for every file descriptor [fd]:
-   * [fd < sysconf_open_max()]
-   *)
-
-external fchdir : Unix.file_descr -> unit = "netsys_fchdir"
-  (** Set the current directory to the directory referenced by the
-      file descriptor
-   *)
-
-external fdopendir : Unix.file_descr -> Unix.dir_handle = "netsys_fdopendir"
-  (** Make a directory handle from a file descriptor. The descriptor
-      is then "owned" by the directory handle, and will be closed by
-      [Unix.closedir].
-
-      This function is useful in conjunction with {!Netsys_posix.openat}
-      to read directories relative to a parent directory.
-
-      This is a recent addition to the POSIX standard; be prepared to
-      get [Invalid_argument] because it is unavailable.
-   *)
-
-(* Process groups, sessions, terminals *)
-
-external getpgid : int -> int = "netsys_getpgid"
-  (** Return the process group ID of the process with the passed PID.
-   * For the number 0, the process group ID of the current process is
-   * returned.
-   *)
-
-val getpgrp : unit -> int
-  (** Same as [getpgid 0], i.e. returns the process group ID of the
-   * current process.
-   *)
-
-external setpgid : int -> int -> unit = "netsys_setpgid"
-  (** [setpgid pid pgid]: Set the process group ID of the process [pid]
-   * to [pgid]. If [pid = 0], the process group ID of the current process
-   * is changed. If [pgid = 0], as process group ID the process ID of the
-   * process referenced by [pid] is used.
-   *
-   * It is only possible for a process to join a process group if both
-   * belong to the same session.
-   *)
-
-val setpgrp : unit -> unit
-  (** Same as [setpgid 0 0]: A new process group ID is created, and the
-   * current process becomes its sole member.
-   *)
-
-external tcgetpgrp : Unix.file_descr -> int = "netsys_tcgetpgrp"
-  (** Return the process group ID of the foreground process group of
-   * the session associated with the file descriptor, which must be
-   * a tty.
-   *)
-
-external tcsetpgrp : Unix.file_descr -> int -> unit = "netsys_tcsetpgrp"
-  (** Sets the foreground process group ID of the session associated
-   * with the file descriptor, which must be a tty.
-   *)
-
-external ctermid : unit -> string = "netsys_ctermid"
-  (** Returns the name of the controlling tty of the current process 
-   * as pathname to a device file
-   *)
-
-external ttyname : Unix.file_descr -> string = "netsys_ttyname"
-  (** Returns the name of the controlling tty referred to by the
-   * file descriptor.
-   *)
-
-external getsid : int -> int = "netsys_getsid"
-  (** Returns the session ID of the process with the passed PID.
-   * For the PID 0, the session ID of the current process is returned.
-   *)
-
-val with_tty : (Unix.file_descr -> unit) -> unit
-  (** [with_tty f]: Runs [f fd] where [fd] is the terminal of the process.
-      If the process does not have a terminal (because it is a daemon) 
-      [with_tty] will fail.
-   *)
-
-val tty_read_password : ?tty:Unix.file_descr -> string -> string
-  (** [tty_read_password prompt]: If [tty] is a terminal, the [prompt]
-      is output, and a password is read from the terminal (echo off).
-      If [tty] is not a terminal, no [prompt] is printed, and just a
-      line is read from the [tty] descriptor (non-interactive case).
-
-      [tty] defaults to [Unix.stdin]. If this function is used in a
-      program where stdin is not redirected, and the program is started
-      in a terminal, it will read the password with prompt and 
-      echo disabled. If stdin is redirected, it is assumed that the program is
-      used in a script, and the password is piped into it.
-
-      Use in conjunction with [with_tty] to ensure that [tty] is
-      the terminal even if a redirection is in effect, e.g.
-      {[ with_tty (fun tty -> tty_read_password ~tty prompt) ]}
-
-      Raises [Sys.Break] if the user triggers SIGINT (i.e. presses
-      CTRL-C) to abort the input of a password.
-   *)
-
-(* Users and groups *)
-
-external setreuid : int -> int -> unit = "netsys_setreuid"
-  (** Changes both the real and the effective user ID of the current
-   * process.
-   *)
-
-external setregid : int -> int -> unit = "netsys_setregid"
-  (** Changes both the real and the effective group ID of the current
-   * process.
-   *)
-
-
 (** {1 Fork+exec} *)
 
 (** The following function has some similarity with posix_spawn, but
-    is changed to our needs, Only special (although frequent) cases
-    can be implemented with posix_spawn.
+    is extended to our needs, Only special (although frequent) cases
+    are implemented with posix_spawn.
  *)
 
 type wd_spec =
   | Wd_keep
+      (** Keep the current working directory in the spawned process *)
   | Wd_chdir of string
+      (** Change to this directory in the spawned process *)
   | Wd_fchdir of Unix.file_descr
+      (** Change to the directory which has been previously been opened *)
 
 type pg_spec =
   | Pg_keep
+      (** The new process will be member of the same process group as
+	  this process *)
   | Pg_new_bg_group
+      (** A new background process group is created, and the spawned
+	  process will be its single member
+       *)
   | Pg_new_fg_group
+      (** A new foreground process group is created, and the spawned
+	  process will be its single member
+       *)
   | Pg_join_group of int
+      (** The spawned process will be member of this process group *)
 
 type fd_action =
   | Fda_close of Unix.file_descr
@@ -382,17 +546,24 @@ type fd_action =
 
 type sig_action =
   | Sig_default of int
+      (** Resets this signal to default behavior in the spawned process *)
   | Sig_ignore of int
+      (** Ignores the signal in the spawned process *)
+  | Sig_mask of int list
+      (** Set the signal mask in the spawned process *)
 
 val spawn : ?chdir:wd_spec ->
             ?pg:pg_spec ->
             ?fd_actions:fd_action list ->
             ?sig_actions:sig_action list ->
             ?env:string array ->
+            ?no_posix_spawn:bool ->
             string -> string array ->
               int
   (** [spawn cmd args]: Fork the process and exec [cmd] which gets the
       arguments [args]. On success, the PID of the new process is returned.
+      This function does not wait for the completion of the process; use
+      [Unix.waitpid] for this purpose.
 
       - [chdir]: If set, the new process starts with this working directory
         (this is done before anything else)
@@ -402,16 +573,141 @@ val spawn : ?chdir:wd_spec ->
       - [sig_actions]: If set, these signal actions are executed sequentially
       - [env]: If set, the process gets this environment instead of the
         current one
+      - [no_posix_spawn]: If set, the [posix_spawn] family of library
+        functions is not used to spawn even if possible, and always a
+        [fork/exec] approach is taken. This may be slower, but there is
+        normally better error reporting.
 
       Any exceptions in the subprocess are detected, and reported. However,
-      if [Fda_close] leads to [EBADF] for a descriptor, this error is
+      if [Fda_close_ignore] leads to [EBADF] for a descriptor, this error is
       ignored.
 
       If [pg=Pg_new_fg_group], one should include [Sig_ignore Sys.sigttou]
       in [sig_actions].
+
+      There are two implementations for [spawn]: One calls [fork] and [exec]
+      directly, and one uses the [posix_spawn] family of library functions.
+      The latter is faster on certain conditions, but this is very OS-specific.
+      A number of features are not supported by [posix_spawn] and will force
+      that [fork/exec] is used: [Wd_chdir], [Wd_fchdir], [Pg_new_fg_group],
+      and [Sig_ignore]. However, note some implementations of [posix_spawn]
+      also fall back to [fork/exec] internally for some combinations of flags,
+      and it is hard to predict which spawn calls can actually be accelerated.
+      The tendency, though, is that recent OS have sped up [posix_spawn]
+      so far possible (e.g. by using [vfork] internally, or even by making
+      [posix_spawn] a system call).
    *)
 
 
+(** {1 Notification via file descriptor events} *)
+
+(** Often, it is advantageous to report asynchronous events via
+    file descriptors. On Linux, this is available via the [eventfd]
+    system call. On other platforms, pipes are used for emulation.
+
+    A [not_event] can have two states: off and on. Initially, the
+    [not_event] is off. By signalling it, the state changes to on,
+    and the underlying real file descriptor becomes readable.
+    By consuming the event, the state is switched back to off.
+
+    Note that a similar API exists for Win32: See {!Netsys_win32.w32_event}.
+ *)
+
+type not_event
+
+val create_event : unit -> not_event
+  (** Creates a new event file descriptor. *)
+
+external set_nonblock_event : not_event -> unit
+  = "netsys_set_nonblock_not_event"
+  (** Sets the event fd to non-blocking mode *)
+
+external get_event_fd : not_event -> Unix.file_descr = "netsys_get_not_event_fd"
+  (** Returns a duplicate of the underlying file descriptor. This should only 
+      be used for one thing: checking whether the desciptor becomes readable.
+      As this is a duplicate, the caller has to close the descriptor.
+   *)
+
+external set_event : not_event -> unit = "netsys_set_not_event"
+  (** Signals the event *)
+
+external wait_event : not_event -> unit = "netsys_wait_not_event"
+  (** If the event fd is not signalled, the function blocks until
+      it gets signalled, even in non-blocking mode.
+   *)
+
+external consume_event : not_event -> unit = "netsys_consume_not_event"
+  (** Consumes the event, and switches the event fd to off again.
+      If the event fd is not signalled, the function blocks until
+      it gets signalled (in blocking mode), or it raises [EAGAIN]
+      or [EWOULDBLOCK] (in non-blocking mode).
+
+      This is effectively an atomic "wait-and-reset" operation.
+   *)
+
+val destroy_event : not_event -> unit
+  (** Releases the OS resources. Note that there can be a hidden second
+      file descriptor, so closing the descriptor returned by [get_event_fd]
+      is not sufficient.
+   *)
+
+val report_signal_as_event : not_event -> int -> unit
+  (** [report_signal_as_event ev sig] Installs a new signal handler for
+      signal [sig] so that [ev] is signalled when a signal arrives.
+   *)
+
+(** {1 Notification queues} *)
+
+(** Unimplemented, but a spec exists. Notification queues are intended
+    for forwarding events from C level to OCaml level. Possible uses:
+    - POSIX timers
+    - Realtime signals
+    - Subprocess monitoring
+    - AIO completion
+ *)
+
+(*
+(** This is a helper data structure only. This type of queue is a FIFO
+    implemented in C. When the queue is filled with data, a notification
+    mechanism is triggered to inform user code. Note that the notification
+    only happens when the first element is added to an empty queue, but not when
+    more elements are added. Also note that there can only be one notification
+    mechanism.
+
+    Only C code can add new elements!
+ *)
+ *)
+
+(*
+type 'a not_queue
+
+val create_nqueue : unit -> 'a not_queue
+  (** create a new notification queue *)
+
+val nqueue_length : 'a not_queue -> int
+  (** returns the number of elements in the queue *)
+
+val nqueue_take : 'a not_queue -> 'a
+  (** takes the front element off the queue and returns it.
+      Raises [Not_found] if the queue is empty
+   *)
+
+val nqueue_reset_notification : 'a not_queue -> unit
+  (** Do not notify *)
+
+val nqueue_notify_via_event : 'a not_queue -> not_event -> unit
+  (** Arranges that the event is signalled when the first element is added
+      to the queue
+   *)
+
+val nqueue_notify_via_condition : 'a not_queue -> Condition.t -> unit
+  (** Arranges that the condition variable is signalled  when the first 
+      element is added to the queue
+   *)
+
+(** Another notification mechanism is described in {!Netsys_posix.sem_not}. *)
+ *)
+
 (** {1 Subprocesses and signals} *)
 
 (** Watching subprocesses requires that the right signal handler is
@@ -625,7 +921,34 @@ external fdatasync : Unix.file_descr -> unit = "netsys_fdatasync"
   (** Syncs only data to disk. If this is not implemented, same effect
       as [fsync]
    *)
- 
+
+(** {1 Sending file descriptors over Unix domain sockets} *)
+
+(** These functions can be used to send file descriptors from one process
+    to another one. The descriptor [sock] must be a connected 
+    Unix domain socket.
+
+    The functionality backing this is non-standard but widely available.
+
+    {b Not yet implemented, but spec exists.}
+ *)
+
+(*
+val have_scm_rights : unit -> bool
+  (** Whether this functionality is available *)
+
+val send_fd : Unix.file_descr -> Unix.file_descr -> unit
+  (** [send_fd sock fd]: Sends [fd] via [sock] as ancillary message.
+      Also sends a single byte 'X' over the main message channel.
+   *)
+
+val receive_fd : Unix.file_descr -> Unix.file_descr
+  (** [receive_fd sock]: Receives a single byte over the main message
+      channel, and checks whether a file descriptor accompanies the
+      byte. If so, it is returned. If not, the function will raise [Not_found].
+   *)
+ *)
+
 (** {1 Optional POSIX functions} *)
 
 external have_fadvise : unit -> bool = "netsys_have_posix_fadvise"
@@ -827,6 +1150,25 @@ val sem_wait : 'kind semaphore -> sem_wait_behavior -> unit
       [sem_wait] may be interrupted by signals.
    *)
 
+(** {2:sem_not Semaphores and notification} *)
+
+(** {b Not yet implemented.} *)
+
+(*
+val nqueue_notify_via_sem : 'a not_queue -> _ semaphore -> unit
+  (** Arranges that the semaphore is signalled (posted)  when the first 
+      element is added to the queue
+   *)
+
+
+val sem_wait_via_event : _ semaphore -> not_event -> unit
+  (** Runs [sem_wait] in a thread, and arranges that the [not_event] is 
+      signalled when 
+      the blocking [sem_wait] succeeds. This function is only available on
+      platforms with [pthread] support, because a helper thread is started.
+   *)
+ *)
+
 (** {1 Locales} *)
 
 type langinfo =
@@ -896,6 +1238,143 @@ val query_langinfo : string -> langinfo
       The value for "" is cached.
    *)
 
+(** {1 Clocks} *)
+
+(** Support for clocks can be assumed to exist on all current POSIX
+    systems.
+ *)
+
+type timespec = float * int
+    (** [(t,t_nanos)]: Specifies a time by a base time [t] to which the
+	nanoseconds [t_nanos] are added.
+
+	If this pair is returned by a function [t] will always be integral.
+	If a pair is passed to a function, it does not matter whether this
+	is the case or not, but using integral values for [t] ensure
+	maximum precision.
+     *)
+
+external nanosleep : timespec -> timespec ref -> unit = "netsys_nanosleep"
+  (** [nanosleep t t_rem]: Sleeps for [t] seconds. The sleep can either be
+      finished, or the sleep can be interrupted by a signal. In the
+      latter case, the function will raise [EÍNTR], and write to [t_rem]
+      the remaining seconds.
+   *)
+
+type clock_id
+
+type clock =
+  | CLOCK_REALTIME        (** A clock measuring wallclock time *)
+  | CLOCK_MONOTONIC       (** A clock measuring kernel time (non-settable). Optional, i.e. not supported by all OS *)
+  | CLOCK_ID of clock_id  (** A clock ID *)
+
+external clock_gettime : clock -> timespec = "netsys_clock_gettime"
+  (** Get the time of this clock *)
+
+external clock_settime : clock -> timespec -> unit = "netsys_clock_settime"
+  (** Set the time of this clock *)
+
+external clock_getres : clock -> timespec = "netsys_clock_getres"
+  (** Get the resolution of this clock *)
+
+external clock_getcpuclockid : int -> clock_id = "netsys_clock_getcpuclockid"
+  (** Return the ID of a clock that counts the CPU seconds of the given
+      process. Pass the PID or 0 for the current process.
+
+      This function is not supported on all OS.
+   *)
+
+(*
+val clock_nanosleep : clock -> bool -> timespec -> timespec ref -> unit
+  (** [clock_nanosleep c abstime t t_rem]: Uses the clock [c] to time
+      a sleep. If [abstime], the function sleeps until [t]. If not
+      [abstime], the function sleeps for [t] seconds.
+
+      The sleep can either be
+      finished, or the sleep can be interrupted by a signal. In the
+      latter case, the function will raise [EÍNTR]. If also [abstime] is
+      not specified, it writes to [t_rem] the remaining seconds.
+
+      This function is not supported on all OS.
+   *)
+ *)
+
+(** {1 POSIX timers} *)
+
+type posix_timer
+
+type timer_expiration =
+  | TEXP_NONE
+  | TEXP_EVENT of not_event
+  | TEXP_EVENT_CREATE
+  | TEXP_SIGNAL of int
+
+(* Future: 
+   TEXP_NQ of ??? not_queue
+   TEXP_THREAD of ??? -> unit
+   TEXP_EVENT_KQUEUE: Like TEXP_EVENT_CREATE, but backed by a kqueue (BSD),
+      and with restrictions (only CLOCK_MONOTONIC, only milliseconds prevision,
+      only oneshot timers)
+ *)
+
+val have_posix_timer : unit -> bool
+
+val timer_create : clock -> timer_expiration -> posix_timer
+  (** Create a new timer that will report expiration as given by the arg:
+      - [TEXP_NONE]: no notification
+      - [TEXP_EVENT e]: the [not_event] [e] is signalled
+      - [TEXP_EVENT_CREATE]: a special [not_event] is created for the timer.
+        (Get the event via [timer_event], see below.)
+      - [TEXP_SIGNAL n]: the signal [n] is sent to the process
+
+      Note that [TEXP_EVENT_CREATE] is much faster on Linux than
+      [TEXP_EVENT], because it can be avoided to start a new thread
+      whenever the timer expires. Instead, the timerfd machinery is used.
+
+      [TEXP_EVENT] and [TEXP_EVENT_CREATE] are only supported on systems
+      with pthreads.
+   *)
+
+val timer_settime : posix_timer -> bool -> timespec -> timespec -> unit
+  (** [timer_settime tm abstime interval value]:
+
+      If [value=(0.0,0)], the timer is stopped.
+
+      If [value] is a positive time, the timer is started (or the timeout
+      is changed if it is already started).  If [abstime], [value] is
+      interpreted as the absolute point in time of the expiration.
+      Otherwise [value] sets the number of seconds until the expiration.
+
+      If [interval] is positive, the started timer will repeat to expire
+      after this many seconds once it has expired for the first time.
+      If [interval=(0.0,0)], the timer is a one-shot timer.
+   *)
+
+val timer_gettime : posix_timer -> timespec
+  (** Returns the number of seconds until the expiration, or [(0.0,0)]
+      if the timer is off
+   *)
+
+val timer_delete : posix_timer -> unit
+  (** Deletes the timer *)
+
+val timer_event : posix_timer -> not_event
+  (** Returns the notification event for the timer styles [TEXP_EVENT] and
+      [TEXP_EVENT_CREATE].
+
+      Note that the latter type of event does not allow to call [set_event].
+   *)
+
+(** Intentionally there is no wrapper for [timer_getoverrun].
+    Additional overruns can occur because of the further processing
+    of the notifications: The OCaml runtime can merge signals,
+    which would not be noticed by the kernel overrun counter,
+    and events can also be merged. The workaround is to use one-shot timers
+    with absolute expiration timestamps, and to check for overruns
+    manually. Once we have [TEXP_NQ] the issue is solved.
+ *)
+
+
 (** {1 Linux I/O Priorities} *)
 
 (** These system calls are only available on Linux since kernel 2.6.13,
