@@ -1,4 +1,4 @@
-(* $Id: rpc_time.ml 1137 2007-07-31 15:48:47Z gerd $
+(* $Id: rpc_time.ml 1196 2008-07-23 01:18:00Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -17,15 +17,17 @@ let remote_time ?(timeout = 5) peer =
     ( try
 	Unix.connect s (Unix.ADDR_INET(peer, 37))
       with
-	  Unix.Unix_error(Unix.EINPROGRESS,_,_) -> ()
+	Unix.Unix_error((Unix.EINPROGRESS|Unix.EWOULDBLOCK),_,_) -> ()
+	  (* Note: Win32 returns EWOULDBLOCK instead of EINPROGRESS *)
     );
     Unix.clear_nonblock s;
     let buf = String.create 4 in
     let pos = ref 0 in
     while !pos < 4 do
-      let r, _, _ = Unix.select [ s ] [] [] (float timeout) in
-      if r = [] then raise Time_not_available;
-      let n = Unix.read s buf !pos (String.length buf - !pos) in
+      let ok = 
+	Netsys.restart_tmo (Netsys.wait_until_connected s) (float timeout) in
+      if not ok then raise Time_not_available;
+      let n = Netsys.blocking_read s buf !pos (String.length buf - !pos) in
       pos := !pos + n;
       if !pos < 4 && n=0 then raise Time_not_available;
     done;
