@@ -1,8 +1,8 @@
-(* $Id: syntax.ml 1444 2010-04-25 23:14:52Z gerd $
+(* $Id: syntax.ml 1701 2012-02-14 14:48:46Z gerd $
  * ----------------------------------------------------------------------
  *)
 
-open Rtypes;;
+open Netnumber;;
 
 exception Error of string;;
 
@@ -91,6 +91,7 @@ and xdr_union =
 and xdr_decl =
     { decl_type : xdr_type;
       decl_symbol : xdr_id;
+      mutable decl_direct : bool;   (* whether direct mapping is possible here *)
     }
 ;;
 
@@ -124,7 +125,8 @@ type xdr_def =
 
 let mk_decl id t =
   { decl_type = t;
-    decl_symbol = id
+    decl_symbol = id;
+    decl_direct = false;   (* initially *)
   }
 ;;
 
@@ -370,6 +372,13 @@ let check_type_constraints dl =
     ()
   in
 
+  let convertible_to_int (sign,v) =
+    (* Only relevant on 32 bit systems *)
+    try ignore(int_of_uint4 v)
+    with _ ->
+      error "The numerical argument is not representable as int"
+  in
+
   let rec get_type t =
     match t with
 	T_refer_to (r,n) ->
@@ -404,12 +413,15 @@ let check_type_constraints dl =
 
   let rec check_type t = (
     match t with
-	T_opaque_fixed c     -> unsigned_int (constant !c)
+	T_opaque_fixed c     -> unsigned_int (constant !c);
+                                convertible_to_int (constant !c)
       | T_opaque c           -> unsigned_int (constant !c)
       | T_string c           -> unsigned_int (constant !c)
       | T_mstring(_,c)       -> unsigned_int (constant !c)
       | T_option t'          -> check_type t'
-      | T_array_fixed(c,t')  -> unsigned_int (constant !c); check_type t'
+      | T_array_fixed(c,t')  -> unsigned_int (constant !c);
+                                convertible_to_int (constant !c);
+                                check_type t'
       | T_array(c,t')        -> unsigned_int (constant !c); check_type t'
       | T_array_unlimited t' -> check_type t'
       | T_refer_to n         -> ignore(get_type t)
